#include "cpu_speed.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define ClearBit(PORT, BIT)   (PORT &= ~(1 << BIT))
#define SetBit(PORT, BIT)     (PORT |= (1 << BIT))
#define FREQ_C1               2   // 1=433MHz, 2=868MHz, 3=915MHz
#define FREQ_C2               43  // 43 = 433MHz/ 868MHz, 30 = 915MHz
#define RF01FREQ(freq)	      (4000*((freq/10*FREQ_C1)-FREQ_C2))

// RFM01 Inputs (= AVR Outputs)
#define RF01_SDI_PORT   PORTB
#define RF01_SDI_DDR    DDRB
#define RF01_SDI        PB3
#define RF01_SCK_PORT   PORTB
#define RF01_SCK_DDR    DDRB
#define RF01_SCK        PB5
#define RF01_NSEL_PORT  PORTB
#define RF01_NSEL_DDR   DDRB
#define RF01_NSEL       PB2

// RFM01 Outputs (= AVR Inputs)
//#define RF01_CLK_PORT   ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF01_CLK_DDR    ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF01_CLK_PIN    ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF01_CLK        ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF01_VDI_PORT   PORTB
//#define RF01_VDI_DDR    DDRB
//#define RF01_VDI_PIN    DDRB
//#define RF01_VDI        PB0
#define RF01_SDO_PORT   PORTB
#define RF01_SDO_DDR    DDRB
#define RF01_SDO_PIN    DDRB
#define RF01_SDO        PB4
//#define RF01_NRES_PORT  ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF01_NRES_DDR   ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF01_NRES_PIN   ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF01_NRES       ???     // Not connected on Pollin FUNK-AVR-Evaluationsboard
//#define RF01_IRQ_PORT   PORTD
//#define RF01_IRQ_DDR    DDRD
//#define RF01_IRQ_PIN    PIND
//#define RF01_IRQ        PD2
//#define RF01_NFFS_PORT   PORTD
//#define RF01_NFFS_DDR    DDRD
//#define RF01_NFFS_PIN    PIND
//#define RF01_NFFS        PD7


// Evaluationsboard Outputs
#define EVA_LED1_PORT   PORTD
#define EVA_LED1_DDR    DDRD
#define EVA_LED1        PD6
#define EVA_LED2_PORT   PORTD
#define EVA_LED2_DDR    DDRD
#define EVA_LED2        PD5

// Evaluationsboard Inputs
#define EVA_BTN1_PORT   PORTB
#define EVA_BTN1_DDR    DDRB
#define EVA_BTN1_PIN    PINB
#define EVA_BTN1        PB1

void rf01_trans(unsigned short wert);
void rf01_init(void);
void rf01_setbandwidth(unsigned char bandwidth);
void rf01_setreceiver(unsigned char gain, unsigned char drssi);
void rf01_setfreq(unsigned short freq);
void rf01_setbaud(unsigned short baud);
void rf01_rxdata(unsigned char *data, unsigned char number);

// nFFS: 1-10k Pullup an Vcc !!!
static unsigned char sdrssi, sgain;

void rf01_trans(unsigned short wert)
{	
  unsigned char i;

	//cbi(RF_PORT, CS);
  ClearBit(RF01_NSEL_PORT, RF01_NSEL);
	for (i=0; i<16; i++)
	{	
    //if (wert&32768)
    if (wert&0x8000)
    {
			//sbi(RF_PORT, SDI);
      SetBit(RF01_SDI_PORT, RF01_SDI);
    }
		else
    {
			//cbi(RF_PORT, SDI);
      ClearBit(RF01_SDI_PORT, RF01_SDI);
    }

		//sbi(RF_PORT, SCK);
    SetBit(RF01_SCK_PORT, RF01_SCK);

		wert<<=1;
    _delay_us(0.2);

		//cbi(RF_PORT, SCK);
    ClearBit(RF01_SCK_PORT, RF01_SCK);
	}
	
  //sbi(RF_PORT, CS);
  SetBit(RF01_SCK_PORT, RF01_SCK);
}

void rf01_init(void)
{	
  //Set used PORTS to output
  RF01_SDI_DDR  |= (1<<RF01_SDI);
  RF01_SCK_DDR  |= (1<<RF01_SCK);
  RF01_NSEL_DDR |= (1<<RF01_NSEL);

  //Set used PORTS to input
  //RF01_IRQ_DDR  &= ~(1<<RF01_IRQ);
  RF01_SDO_DDR  &= ~(1<<RF01_SDO);

  //Set output ports
  RF01_NSEL_PORT |= (1<<RF01_NSEL);

  //Enable pullups at input pins
  //RF01_IRQ_PORT  |= (1<<RF01_IRQ);
  //RF01_SDO_PORT  |= (1<<RF01_SDO);


	for (unsigned char i=0; i<15 ; i++)
		_delay_ms(10);			// wait until POR done

	rf01_trans(0xC2E0);			// AVR CLK: 10MHz
	rf01_trans(0xC42B);			// Data Filter: internal
	rf01_trans(0xCE88);			// FIFO mode
	rf01_trans(0xC6F7);			// AFC settings: autotuning: -10kHz...+7,5kHz
	rf01_trans(0xE000);			// disable wakeuptimer
	rf01_trans(0xCC00);			// disable low duty cycle
}

void rf01_setbandwidth(unsigned char bandwidth)
{
	//rf01_trans(0x8970|((bandwidth&7)<<1));  // 433MHz
  rf01_trans(0x9170|((bandwidth&7)<<1));  // 868MHz
}

void rf01_setreceiver(unsigned char gain, unsigned char drssi)
{
	sdrssi=drssi;
	sgain=gain;
}

void rf01_setfreq(unsigned short freq)
{	if (freq<96)				// 430,2400MHz | 860,4800MHz | 900,7200MHz
		freq=96;
	else if (freq>3903)			// 439,7575MHz | 879,5100MHz | 929,2700MHz
		freq=3903;
	rf01_trans(0xA000|freq);
}

void rf01_setbaud(unsigned short baud)
{
	if (baud<336)
		return;
	if (baud<5400)				// Baudrate= 344827,58621/(R+1)/(1+CS*7)
		rf01_trans(0xC880|((43104/baud)-1));
	else
		rf01_trans(0xC800|((344828UL/baud)-1));
}


void rf01_rxdata(unsigned char *data, unsigned char number)
{	
  unsigned char i,j,c;

	rf01_trans(0xC0C1|((sgain&3)<<4)|((sdrssi&7)<<1));	// RX on
	rf01_trans(0xCE89);			// set FIFO mode
	rf01_trans(0xCE8B);			// enable FIFO
	
  //cbi(RF_PORT, SDI);
  ClearBit(RF01_SDI_PORT, RF01_SDI);

	for (i=0; i<number; i++)
	{
  	//cbi(RF_PORT, CS);
    ClearBit(RF01_NSEL_PORT, RF01_NSEL);

		//while (!(RF_PIN&(1<<SDO))); // wait until data in FIFO
    while (!(RF01_SDO_PIN&(1<<RF01_SDO)))
    {
      // wait until data in FIFO
      EVA_LED2_PORT ^= (1<<EVA_LED2);
    }

		for (j=0; j<16; j++)	// read and discard status register
		{	
      //sbi(RF_PORT, SCK);
      SetBit(RF01_SCK_PORT, RF01_SCK);
			asm("nop");
			//cbi(RF_PORT, SCK);
      ClearBit(RF01_SCK_PORT, RF01_SCK);
		}

		c=0;
		for (j=0; j<8; j++)
		{
      c<<=1;
			//if (RF_PIN&(1<<SDO))
			//	c|=1;
			if (RF01_SDO_PIN&(1<<RF01_SDO))
      {
				c|=1;
      }

      //sbi(RF_PORT, SCK);
      SetBit(RF01_SCK_PORT, RF01_SCK);
			_delay_us(0.2);
			//cbi(RF_PORT, SCK);
      ClearBit(RF01_SCK_PORT, RF01_SCK);
		}

		*data++=c;
		//sbi(RF_PORT, CS);
    ClearBit(RF01_NSEL_PORT, RF01_NSEL);
	}

	rf01_trans(0xC0C0|((sgain&3)<<4)|((sdrssi&7)<<1));	// RX off
}

unsigned char data[32];

int main(void)
{
  SetBit(EVA_LED1_PORT, EVA_LED1);
  SetBit(EVA_LED1_DDR,  EVA_LED1);
  SetBit(EVA_LED2_PORT, EVA_LED2);
  SetBit(EVA_LED2_DDR,  EVA_LED2);

  char cKanal = 4; // Die Kanle 4, 7, 10 und 13 drfen ohne Verwendung von LBT genutzt werden (also 100% Senden)

	rf01_init();					// ein paar Register setzen (z.B. CLK auf 10MHz)
	//rf01_setfreq(RF01FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen
  rf01_setfreq(RF01FREQ(864.9 + (0.2*cKanal)));	// Sende/Empfangsfrequenz auf 875MHz einstellen
	rf01_setbandwidth(4);			// 200kHz Bandbreite
	rf01_setreceiver(2,4);			// -6dB Verstrkung, DRSSI threshold: -79dBm 
	rf01_setbaud(19200);			// 19200 Baud

	for (;;)
	{	
    //rf01_rxdata(data, 32);		// 32Bytes empfangen
		// hier die Daten verarbeiten
    //EVA_LED1_PORT ^= (1<<EVA_LED1);


    PORTD &= ~(1<<PD6);
    PORTD |= (1<<PD6);

	}
}
